/***********************************************************
 * $Id: fancysockwsframeheader.h 1772 2016-08-07 20:15:28Z fancylibs $
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYSOCKWSFRAMEHEADER_H_
#define FANCYSOCKWSFRAMEHEADER_H_

#include <fancysock.h>
#include <fancyobject.h>

namespace Fancy {

class Channel;
template<class T> class ArrayObject;

namespace Sock {

/**
 * A Web Socket frame header as defined in
 * <a href="https://tools.ietf.org/html/rfc6455">RFC 6455</a>.
 */
class FANCYSOCK_API WsFrameHeader: public virtual Fancy::Object {

    bool fin;
    int opcode;
    Fancy::ObjPtr<Fancy::ArrayObject<int8_t> > maskingKey;
    int64_t payloadLength;


protected:
    WsFrameHeader(bool _fin, int _opcode, const Fancy::ObjPtr<Fancy::ArrayObject<int8_t> > &_maskingKey, int64_t _payloadLength);
	virtual ~WsFrameHeader();

public:
    static const int opCodeContinuation = 0x0;
    static const int opCodeText         = 0x1;
    static const int opCodeBinary       = 0x2;
    static const int opCodeClose        = 0x8;
    static const int opCodePing         = 0x9;
    static const int opCodePong         = 0xa;

    typedef Fancy::ObjPtr<WsFrameHeader> ptr_t;

    /**
     * Parse a new frame from an input stream.
     */
    static ptr_t newInstance(const Fancy::ObjPtr<Channel> &channel);

    /**
     * Write this frame to an output stream.
     */
    void serialize(const Fancy::ObjPtr<Channel> &channel);

    /**
     * @param opcode An opcode to check.
     * @return Whether the given number is a valid opcode.
     *
     * @see #opCodeContinuation
     * @see #opCodeText
     * @see #opCodeBinary
     * @see #opCodeClose
     * @see #opCodePing
     * @see #opCodePong
     */
    static bool isValidOpCode(int opcode);

    /**
      * @param opcode An opcode to check.
      * @return Whether the given number is a valid control opcode.
      *
      * @see #opCodeClose
      * @see #opCodePing
      * @see #opCodePong
      */
    static bool isControlOpCode(int opcode);

    /**
     * @return The fin flag, which may only be <code>false</code> for
     *            messages with an opcode of {@link #opCodeBinary},
     *             {@link #opCodeText} or {@link #opCodeContinuation}.
     */
    bool isFin() const {
        return this->fin;
    }

    /**
     * @return The 32-bit masking key or <code>null</code>, if the message
     *         has not been masked.
     */
    const Fancy::ObjPtr<Fancy::ArrayObject<int8_t> >& getMaskingKey() const {
        return this->maskingKey;
    }

    /**
     * @return The message opcode.
     * @see #opCodeContinuation
     * @see #opCodeText
     * @see #opCodeBinary
     * @see #opCodeClose
     * @see #opCodePing
     * @see #opCodePong
     */
    int getOpcode() const {
        return opcode;
    }

    /**
     * @return The parsed length of the payload.
     */
    int64_t getPayloadLength() const {
        return this->payloadLength;
    }
};

}

}

#endif /* FANCYSOCKWSFRAMEHEADER_H_ */
